import sys

import io
import argparse

from threading import Thread, Event, Lock
from multiprocessing import Queue
import queue
from os import fdopen, path

from . import argparser_wrapper
from . import views
from . import context


class QueuedOut(io.StringIO):
    def __init__(self, name, shared_queue, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name = name
        self.queue = shared_queue

    def write(self, b, *args, **kwargs):
        # flush at newline
        lines = b.split('\n')
        for line in lines[:-1]:
            super().write(line)
            self.flush()
        super().write(lines[-1])

    def flush(self):
        value = self.getvalue()
        if len(value) > 1:
            self.queue.put((self.name, value))
            self.seek(0)
            self.truncate(0)

class FlaskThread(Thread):
    def __init__(self, port, host):
        super().__init__()
        self.port = port
        self.host = host

    def run(self):
        views.app.run(port=self.port, threaded=True, host=self.host)

class OutputThread(Thread):
    def __init__(self, inqueue, restart):
        super().__init__()
        self.queue = inqueue
        self.restart = restart
        self.cache = []
        self.clients = []
        self.sem = Lock()
        self.stopped = False

    def stop(self):
        self.stopped = True

    def run(self):
        while not self.restart.is_set() or self.stopped:
            item = self.queue.get()
            self.cache.append(item)
            self.sem.acquire()
            for i, (outqueue, active) in enumerate(self.clients):
                try:
                    outqueue.put(item, True, 1)
                except queue.Full:
                    self.clients[i] = self.clients[i][0], False
            self.sem.release()

            self.clients = [client for client in self.clients if client[1] == True]

    def add_client(self):
        print("new connection. Current connections: %s" % len(self.clients))
        new_queue = queue.Queue(10)
        self.clients.append((new_queue, True))
        return new_queue


def start_module(name, is_module):
    views.app.restart.clear()
    views.app.name = path.basename(name)
    views.app.desc = ""
    views.app.actions = []
    views.app.queue = Queue()
    ioout = QueuedOut("out", views.app.queue)
    ioerr = QueuedOut("err", views.app.queue)
    views.app.output = OutputThread(views.app.queue, views.app.restart)
    #Output(views.app.queue, views.app.restart)

    views.app.actionQueue = Queue()  # This holds only one Argparser Object
    views.app.namespaceQueue = Queue()  # This hold only one Namespace Object

    argparser = argparser_wrapper.argParserGenerator(views.app.actionQueue, views.app.namespaceQueue)

    views.app.module_process = context.Context(
        "sub",
        name,
        ioout,
        ioerr,
        overwritten_modules={'argparse': argparser},
        is_module = is_module,
        #original_modules = emptymodules
    )
    views.app.module_process.start()
    views.app.output.start()
    views.app.mutex_groups, views.app.actions, name, views.app.desc = views.app.actionQueue.get()
    if name:
        views.app.name = name

    views.app.module_process.join()
    views.app.output.stop()

    ioerr.write("Process stopped ({})\n".format(views.app.module_process.exitcode))
    views.app.restart.wait()


def main():

    parser = argparse.ArgumentParser(description="a Webbased frontend for ARgparse in Python")
    parser.add_argument('--port', '-p', default=5000, help="The port to listen on (default 5000)")
    parser.add_argument('--host', default="0.0.0.0", help="The host to bind to (default 0.0.0.0)")
    parser.add_argument('--module', '-m', action="store_true", help="If set, loads a module instead of a file")
    parser.add_argument('file', help="File to run")
    args = parser.parse_args()

    flask_thread = FlaskThread(args.port, args.host)
    flask_thread.start()

    views.app.restart = Event()
    while True:
        start_module(args.file, args.module)


if __name__ == "__main__":
    main()



import argparse
import sys
import io
import runpy
from traceback import print_exc
from contextlib import redirect_stdout, redirect_stderr

from multiprocessing import Process

class Context(Process):
    def __init__(self, name, path, stdout, stderr, arguments=[], is_module = False, overwritten_modules={}, original_modules=None):
        super().__init__()
        self.name = name
        self.stdout = stdout
        self.stderr = stderr
        self.path = path
        self.is_module = is_module
        self.arguments = arguments
        self.overwritten_modules = overwritten_modules
        self.original_modules = original_modules

    def run(self):
        if self.arguments is not None:
            sys.argv = [''] + self.arguments
        else:
            sys.argv = ['']
        #if not self.original_modules is None:
        #    sys.modules = self.original_modules
        #    print(sys.modules)
        sys.modules.update(self.overwritten_modules)
        with redirect_stdout(self.stdout):
            with redirect_stderr(self.stderr):
                try:
                    if self.is_module:
                        runpy.run_module(self.path, run_name='__main__')
                    else:
                        runpy.run_path(self.path, run_name='__main__')
                except Exception:
                    print_exc(file=sys.stderr)

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--path")
    parser.add_argument("--arguments", '-a',  default=None)
    parser.add_argument("--is-module", "-m", default=False, action="store_true", dest="is_module")
    args = parser.parse_args()

    context = Context("test", args.path, sys.stdout, sys.stderr, None if args.arguments is None else [args.arguments], args.is_module)
    context.start()



from uuid import uuid4





class Action():
    def identity(self, x):
        if x is None:
            return self.on_none
        return x

    def __init__(self, action, **kwargs):
        self.uuid = uuid4()
        self.dest = action.dest
        try:
            self.name = action.name
        except AttributeError:
            self.name = action.dest
        self.desc = action.help
        self.on_set = action.const
        self.on_none = action.default
        if action.option_strings \
                and not action.required \
                and action.const is None \
                and action.nargs != '*'\
                or action.nargs == '?':
            self.optional = True
        else:
            self.optional = False
        # self.type_function = action.type if action.type is not None else self.identity
        self.type_function = action.type if action.type is not None else str
        self.const = action.const
        self.nargs = action.nargs

    def __repr__(self):
        return self.name

    def as_dict(self):
        internal_dict = {}
        internal_dict['uuid'] = str(self.uuid)
        internal_dict['name'] = self.name
        internal_dict['dest'] = self.dest
        internal_dict['desc'] = self.desc
        internal_dict['checked'] = self.on_none if self.on_none is True or self.on_none is False else None
        internal_dict['default'] = self.on_none if type(self.on_none) == str else None
        internal_dict['optional'] = self.optional
        internal_dict['is_const'] = self.const is not None
        internal_dict['type'] = self.type_function.__name__
        internal_dict['nargs'] = self.nargs
        try:
            internal_dict['choices'] = {key: ac.as_dict() for key, ac in self.choices.items()}
        except AttributeError:
            pass
        return internal_dict


class AppendAction(Action):
    def __init__(self, action):
        super().__init__(action)
        if self.optional:
            self.nargs = '*'
        else:
            self.nargs = '+'

class SubParserAction(Action):
    def __init__(self, action):
        super().__init__(action)
        self.choices = {}
        for name, subParser in action.choices.items():
            self.choices[name] = ActionContainer(name)
            actions, mutex_groups = subParser.get_actions()
            self.choices[name].actions = actions
            self.choices[name].groups = mutex_groups

    def __repr__(self):
        return "SubParsers: %s" % self.choices

class ActionContainer():
    def __init__(self, name=None):
        self.name = name
        self.actions = []
        self.groups = []
        self.uuid = uuid4()

    def __repr__(self):
        return "ActionContainer(actions: %s, groups: %s)" % (self.actions, self.groups)

    def as_dict(self):
        internal_dict = {}
        internal_dict['uuid'] = str(self.uuid)
        internal_dict['actions'] = [action.as_dict() for action in self.actions]
        internal_dict['groups'] = [[action.as_dict() for action in group] for group in self.groups]
        return internal_dict

class StoreAction(Action):
    pass

class StoreConstAction(Action):
    def store_type_function(self, x):
        return self.const if x is not None else self.on_none

    def __init__(self, action, **kwargs):
        super().__init__(action, **kwargs)
        self.type_function = self.store_type_function


class MutuallyExclusiveGroup(ActionContainer):
    def __init__(self):
        super().__init__()

    def __repr__(self):
        return "Group Object: ( Actions: {}, Groups: {} )".format(self.actions, self.mutex_groups)



import argparse
import os
import sys
from types import ModuleType

from . import actions

def argParserGenerator(actionQueue, namespaceQueue):
    class MyArgParser(argparse.ArgumentParser):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.actionQueue = actionQueue
            self.namespaceQueue = namespaceQueue

        def get_actions(self):
            actions = []
            mutex_groups = []

            # First add all actions (Including actions of mutex_groups)
            for action in self._actions:
                if type(action) != argparse._HelpAction:
                    actions.append(self.simplify_action(action))


            # Then fill all mutex groups
            mutex_groups = self.create_mutex_groups(self._mutually_exclusive_groups, self)

            # And remove Actions, that are somewhere in the mutex groups
            actions = self.clean_actions(actions, mutex_groups)

            return actions, mutex_groups


        def simplify_action(self, action):
            if type(action) == argparse._HelpAction:
                return None
            elif type(action) == argparse._StoreConstAction:
                return actions.StoreConstAction(action)
            elif type(action) == argparse._StoreTrueAction:
                return actions.StoreConstAction(action)
            elif type(action) == argparse._StoreFalseAction:
                return actions.StoreConstAction(action)
            elif type(action) == argparse._StoreAction:
                return actions.StoreAction(action)
            elif type(action) == argparse._AppendAction:
                return actions.AppendAction(action)
            elif type(action) == argparse._CountAction:
                return actions.StoreAction(action, type_function=int)
            elif type(action) == argparse._SubParsersAction:
                return actions.SubParserAction(action)
            print("Unknown type: {}".format(type(action)), file=sys.__stderr__)
            return None

        def create_mutex_groups(self, groups, parent):
            mutex_groups = []
            for group in groups:
                if group._container == parent:
                    new_group = actions.MutuallyExclusiveGroup()
                    new_group.mutex_groups = self.create_mutex_groups(groups, group)
                    for action in group._group_actions:
                        new_group.actions.append(self.simplify_action(action))
                    mutex_groups.append(new_group)
            return mutex_groups

        def clean_actions(self, actions, mutex_groups):
            for mutex_group in mutex_groups:
                actions = [action for action in actions if action.name not in [act.name for act in mutex_group.actions]]
            return actions

        def parse_args(self, args=None, namespace=None):
            name = os.path.basename(sys.argv[0])
            desc = self.description

            actions, mutex_groups = self.get_actions()

            self.actionQueue.put((mutex_groups, actions, name, desc))
            namespace = self.namespaceQueue.get()
            return namespace
    module = ModuleType('argparse', 'Argument Parser')
    module.__dict__.update(argparse.__dict__)
    module.__dict__['ArgumentParser'] = MyArgParser
    return module




import sys
import signal
import os
import json
import argparse
from datetime import datetime

from flask import Flask, render_template, request, Response, \
    jsonify, send_file, make_response

# Initialize global variables
app = Flask(__name__)
app.mutex_groups=[]

def parse_argument(name, json, action):
    try:
        argument = json[name]
        if type(argument) == list:
            if len(argument) == 1:
                return action.type_function(argument[0])
            else:
                return [action.type_function(elem) for elem in argument]
        else:
            return action.type_function(argument)
    except (KeyError, ValueError):
        if name.endswith("[]"):
            return action.on_none
        else:
            return parse_argument(name + "[]", json, action)


@app.route("/arguments", methods=['POST'])
def fill_namespace():
    json = dict(request.form)
    namespace = argparse.Namespace()
    all_actions = app.actions

    for group in app.mutex_groups:
        all_actions.extend(group.actions)

    for action in all_actions:
        value = parse_argument(action.name, json, action)
        setattr(namespace, action.name, value)

    app.namespaceQueue.put(namespace)
    app.output.queue.put(("sig", "start"))
    return "OK"


@app.route("/arguments", methods=['GET'])
def get_arguments():
    return jsonify({'actions': [a.as_dict() for a in app.actions],
                    'groups': [g.as_dict() for g in app.mutex_groups]})


@app.route("/stop")
def stop():
    os.kill(app.module_process.pid, signal.SIGCONT)
    app.module_process.terminate()
    app.output.queue.put(("sig", "stop"))
    return "OK"

@app.route("/resume")
def resume():
    os.kill(app.module_process.pid, signal.SIGCONT)
    app.output.queue.put(("sig", "resume"))
    return "OK"

@app.route("/pause")
def pause():
    os.kill(app.module_process.pid, signal.SIGSTOP)
    app.output.queue.put(("sig", "pause"))
    return "OK"

@app.route('/clear')
def clear():
    app.output.queue.put(("sig", "clear"));
    return "OK"

@app.route("/reload")
def reload():
    if app.module_process.is_alive():
        return 403, "Process is still running"
    app.restart.set()
    app.output.queue.put(("sig", "reload"))
    return "OK"

@app.route("/download", methods=['GET'])
def download():
    output = "\n".join([line for msg_type, line in app.output.cache if msg_type != 'sig'])
    response = make_response(output)
    response.headers["Content-Disposition"] = \
        "attachment; filename=%s_%s.log" \
        % (app.name, datetime.now().strftime('%Y%m%d-%H%M%S'))
    return response


@app.route("/output.json", methods=['GET'])
def output():
    def generate():
        yield '{"output":['
        app.output.sem.acquire()
        cache = app.output.cache
        output = app.output.add_client()
        app.output.sem.release()
        for msg_type, line in cache:
            yield json.dumps({'type': msg_type, 'line': line})
            yield ','

        while not app.restart.is_set():
            msg_type, line = output.get()
            print("Send ({}): {} (length: {})".format(msg_type, line, len(line)), file=sys.__stdout__)
            yield json.dumps({'type' : msg_type, 'line': line})
            yield ','

        yield '\{\}]}'

    return Response(generate(), mimetype="application/json")



@app.route("/", methods=['GET'])
def index():
    return render_template("index.html", mutex_groups=app.mutex_groups, actions=app.actions, name=app.name, description=app.desc)




import argparse

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest="command")
    first = subparsers.add_parser("first")
    first.add_argument("-f")
    second = subparsers.add_parser("second")
    first.add_argument("-g")

    args = parser.parse_args()

    print("Subparser %s was selected" % args.command)



from argparse import ArgumentParser
import multiprocessing

import time
import sys

def f():
    print("Bla from different Process")


def main():
    print("first")
    print("second")
    parser = ArgumentParser()
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-a')
    group.add_argument('-b')
    group2 = group.add_mutually_exclusive_group()
    group2.add_argument('-c')
    group2.add_argument('-d')
    group3 = group2.add_mutually_exclusive_group()
    group3.add_argument('-e')
    group3.add_argument('-f')

    args = parser.parse_args()

main()



from setuptools import setup, find_packages

version = "0.1.1"

setup(
    name='warped',
    version=version,
    packages=find_packages(),
    url='https://git.k-fortytwo.de/christofsteel/warp',
    download_url = 'https://git.k-fortytwo.de/christofsteel/warped/archive/%s.tar.gz' % version,
    license='MIT',
    author='Christoph Stahl',
    author_email='christoph.stahl@uni-dortmund.de',
    description='A webbased wrapper for the argument parser in Python',
    entry_points={
        'console_scripts': [
            "warped = warped.hook:main"
        ]
    },
    include_package_data = True,
    zip_safe=False,
    install_requires = ['Flask']
)



from argparse import ArgumentParser
import multiprocessing

import time
import sys

def f():
    print("Bla from different Process")


def main():
    print("first")
    print("second")
    bla = ArgumentParser()
    group = bla.add_mutually_exclusive_group()
    group.add_argument('--foo', action='store_true')
    group.add_argument('--bar', action='store_false')
    group2 = group.add_mutually_exclusive_group()
    group2.add_argument('--one')
    group2.add_argument('--two')
    group3 = group2.add_mutually_exclusive_group()
    group3.add_argument('--three')
    group3.add_argument('--four')
    bla.add_argument('--test', '-t', help="Blubb", default="Blabla")
    bla.add_argument('--bla', action="store_true", default=True)
    bla.add_argument('--blubb', action="store_true")
    bla.add_argument('-f', action="append")
    bla.add_argument('Numbers', nargs="*", type=int)
    bla.add_argument('-v', action="count", help="Verbosity Level")
    bla.add_argument('--sum', action="store_const", const=sum, default=max, help="blubb")

    sp = bla.add_subparsers()

    blip = sp.add_parser("blip")
    blip.add_argument("-g", nargs="+")
    blue = sp.add_parser("blue")
    blue.add_argument("-g")
    blue.add_argument("-q")
    subsub = blue.add_subparsers()
    sub1 = subsub.add_parser("sub1")
    sub1.add_argument("-X")
    sub2 = subsub.add_parser("sub2")
    sub2.add_argument("-X")
    nocheins = sp.add_parser("nocheins")
    nocheins.add_argument("-q", action="store_true")

    bla.add_argument('last')

    args = bla.parse_args()
    print(args)

    print(args.sum([1,2,3]))
    print(args)
    print("Blabla", flush=True)
    print(__name__)
    bla.print_help()
    print("Testfehler", file=sys.stderr)

    p = multiprocessing.Process(target=f)
    p.start()
    p.join()

    while True:
        time.sleep(1)
        print("Output")

if __name__ == "__main__":
    main()

